import io

f = io.FileIO('test.xml')
br = io.BufferedReader(f)
print(br.read())

binary_stream_from_file =io.BufferedReader(io.BytesIO(b'starship.png'))
bytes = binary_stream_from_file.read(4)
print(bytes,'\n')

f = io.FileIO('test.xml')
br = io.BufferedReader(f)
text_stream = io.TextIOWrapper(br, 'utf-8')
print('text_stream.readable():', text_stream.readable())
print('text_stream.seekable()', text_stream.seekable())
print('text_stream.writeable()', text_stream.writable())
print(text_stream.read())
text_stream.close()




in_memory_text_stream = io.StringIO('to be or not to be that is the question')
print('in_memory_text_stream', in_memory_text_stream)
print(in_memory_text_stream.getvalue())
in_memory_text_stream.close()

print()




import argparse

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Echo your input')
    parser.add_argument('message',
                        help='Message to echo')

    parser.add_argument('--aaaa', '-a',
                        help='AAA a A',
                        action='store_true')
    parser.add_argument('--bbbb', '-b',
                        help='Do it twice',
                        action='store_false')

    parser.add_argument('--cccc', '-c',
                        help='Do it twice',
                        action='store')

    parser.add_argument('--dddd', '-d',
                        help='Do it twice',
                        action='store_const',
                        const='lll')

    args = parser.parse_args()

    print(args)

    print(args.message)
    if args.aaaa:
        print(args.message)

    if args.bbbb:
        print('this is the shit i told')

    if args.cccc:
        print(args.cccc)

    print(args.dddd)


from openpyxl import Workbook, load_workbook

wb = Workbook()
ws = wb.active  # get current active sheet
print(ws)
ws = wb.create_sheet('Mysheet')  # create a new sheet with title Mysheet
ws.title = 'first title'  # changes the title to first title
ws.sheet_properties.tabColor = "1072BA"



ws['A1'] = 42
ws['A2'] = 40
cell1 = ws['A1']
print(cell1.value)

d = ws.cell(row=4, column=2, value=10)
print(d.value)

ws.append([1, 2, 3]) # appends this list to fifth row because of cursor state
cell_range = ws['A1':'C2']

print(cell_range)
print([j.value for i in cell_range for j in i])

col = ws['C']
col_range = ws['C:D']
row10 = ws[10]
row_range = ws[5:10]

ws['A3'] = '=SUM(A1, A2)'

wb.save('balances.xlsx')


workbook = load_workbook(filename='test.xlsx')
print(workbook.sheetnames)
print(workbook.worksheets)
ws = workbook['فاتحیC1']
print(ws['b7'].value)

for i in range(4,27):
    print(ws['B'+str(i)],'    ',ws['B'+str(i)].value)


import os
from datetime import date, datetime
import zlib

# datetime-------------------------------------------------
now = date.today()
print(now)
print(now.strftime("%m-%d-%y. %d %b %Y is a %A on the %d day of %B."))
birthday = date(1987, 7, 31)
birthday2 = datetime(1987, 7, 31)
x = now - birthday
print(str(x.days // 365) + ' years')

s = b'witch which has which witches wrist watch'
s3 = 'witch which has which witches wrist watch'
print(type(s))



# compression------------------------------------------------
t = zlib.compress(s)
print(t)
s2 = zlib.decompress(t)
print(s == s2)
print(s2)
print(type(s2))
print(s)
print(zlib.crc32(s))

# Timer------------------------------------------------------
from timeit import Timer

traditional_swap_time = Timer('t=a; a=b; b=t', 'a=1; b=2').timeit()

print(traditional_swap_time)
tuple_unpacking_swap_time = Timer('a,b = b,a', 'a=1; b=2').timeit()
print(tuple_unpacking_swap_time)

print(Timer('a**b', 'a=5;b=26').timeit())
print(Timer('a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a', 'a=5;').timeit())

# Test Example------------------------------------------------------
def average(values):
    """Computes the arithmetic mean of a list of numbers.

    >>> print(average([20, 30, 70]))
    40.0
    """
    return sum(values) / len(values)

import doctest
doctest.testmod()
def power2(base, power3):
    """
    >>> print(power2(3,5))
    243
    """
    x = base
    for i in range(power3-1):
        x = base * x

    return x


import doctest

doctest.testmod()

# strings split and list join-------------------------------
strings='problem1, problem2, problem3, problem4, problem5, problem6'
list1=strings.split(', ')
print(list1)
print(' and '.join(list1))


# directories------------------------------------------------
print(os.stat('conf.yaml'))

cur=os.getcwd()
print(os.path.split(cur))
print(os.path.dirname(cur))
print(os.path.basename(cur))
def walk_path(parent_path):
    for parent_path, directories, files in os.walk(parent_path):
        print(f"Checking: {parent_path}")
        for file_name in files:
            file_path = os.path.join(parent_path, file_name)
            last_access = os.path.getatime(file_path)
            size = os.path.getsize(file_path)
            print(f"File: {file_path}")
            print(f"\tlast accessed: {last_access}")
            print(f"\tsize: {size}")

dir = os.getcwd()
walk_path(dir)

from pathlib import Path

pathobj=Path.cwd()

string1=pathobj.as_posix()

print(type(string1))


file_path = Path(__file__).resolve()
parent_path = file_path.parent

print(file_path)
print(file_path.absolute())
print(__file__)


import logging


logging.basicConfig(level=logging.DEBUG)  # or logger.setLevel(logging.DEBUG)
# THIS LINE MEANS LOGGER MUST SHOW THIS LEVEL AND HIGHER

logger = logging.getLogger()
# logger.disabled = True

logger.debug('This is to help with debugging') # 10
logger.info('This is just for information') # 20
logger.log(logging.INFO, "This is just for information")
logger.warning('This is a warning!') # 30
logging.warning('This is a warning!') # 30
logger.error('This should be used with something unexpected') # 40
logger.critical('Something serious') # 50

try:
    x = 1 / 0
except:
    logging.exception('an exception message')
    #logger.exception('an exception message')

print(logger.getEffectiveLevel())
print(logger.isEnabledFor(level=logging.NOTSET))
print(logger.isEnabledFor(level=logging.DEBUG))



logger = logging.getLogger()
print('Root logger:', logger)
logger1 = logging.getLogger('my logger')
print('Named logger:', logger1)
logger2 = logging.getLogger(__name__)
print('Module logger:', logger2)


logger.warning('%s is set to %d', 'count', 42)

# set handler and formatter---------------------------------------------
file_handler = logging.FileHandler('detailed.log')

formatter = logging.Formatter('%(asctime)s [%(levelname)s] %(name)s.%(funcName)s: %(message)s',
                              datefmt='%m-%d-%Y %I:%M:%S|%p')
# mongh day year level name fname message

file_handler.setFormatter(formatter)
# Add the handler to the Logger
logger.addHandler(file_handler)
logger.info('Starting')

#filters-------------------------------------------------
class MyFilter(logging.Filter):
    def filter(self, record):
        if 'John' in record.msg:
            return False
        else:
            return True
logging.basicConfig(format='%(asctime)s %(message)s',
level=logging.DEBUG)
logger = logging.getLogger()
logger.addFilter(MyFilter())
logger.debug('This is to help with debugging')
logger.info('This is information on John')




import turtle
#_________________________________________________________
IMAGE_SIZE_Y=500
IMAGE_SIZE_X=500
MAX_Y=500
MAX_X=500
MIN_X=10
MIN_Y=10
SCREEN_OFFSET_X=25
SCREEN_OFFSET_Y=25
MAX_ITERATIONS=5

for y in range(IMAGE_SIZE_Y):
    zy = y * (MAX_Y - MIN_Y) / (IMAGE_SIZE_Y - 1) + MIN_Y
    for x in range(IMAGE_SIZE_X):
        zx = x * (MAX_X - MIN_X) / (IMAGE_SIZE_Y - 1) + MIN_X
        z = zx + zy * 1j
        c = z
        for i in range(MAX_ITERATIONS):
            if abs(z) > 2.0:
                break
            z = z * z + c
        turtle.color((i % 4 * 64, i % 8 * 32, i % 16 * 16))
        turtle.speed(10)
        turtle.setposition(x - SCREEN_OFFSET_X,
        y - SCREEN_OFFSET_Y)
        turtle.pendown()
        turtle.dot(1)
        turtle.penup()

#_______________________________________________________________
import turtle
# Set up Constants
ANGLES = [60, -120, 60, 0]
SIZE_OF_SNOWFLAKE = 300
def get_input_depth():
    """ Obtain input from user and convert to an int"""
    message = 'Please provide the depth (0 or a positive interger):'
    value_as_string = input(message)
    while not value_as_string.isnumeric():
        print('The input must be an integer!')
        value_as_string = input(message)
    return int(value_as_string)
def setup_screen(title, background='white', screen_size_x=640,
screen_size_y=320, tracer_size=800):
    print('Set up Screen')
    turtle.title(title)
    turtle.setup(screen_size_x, screen_size_y)
    turtle.hideturtle()
    turtle.penup()
    turtle.backward(240)
    # Batch drawing to the screen for faster rendering
    turtle.tracer(tracer_size)
    turtle.bgcolor(background) # Set the background colour of the screen

def draw_koch(size, depth):
    if depth > 0:
        for angle in ANGLES:
            draw_koch(size / 3, depth - 1)
            turtle.left(angle)
    else:
        turtle.forward(size)
depth = get_input_depth()
setup_screen('Koch Snowflake (depth ' + str(depth) + ')',
background='black',
screen_size_x=420, screen_size_y=420)
# Set foreground colours
turtle.color('sky blue')
# Ensure snowflake is centred
turtle.penup()
turtle.setposition(-180,0)
turtle.left(30)
turtle.pendown()
# Draw three sides of snowflake
for _ in range(3):
    draw_koch(SIZE_OF_SNOWFLAKE, depth)
    turtle.right(120)
# Ensure that all the drawing is rendered
turtle.update()
print('Done')
turtle.done()
#_____________________________________________________

import fileinput
import glob

with open('test.xml', 'r+') as f:
    print(f.name)
    print(f.mode)
    print(f.closed)
print(f.closed)


with fileinput.input(files=('test.xml','test2.xml')) as f:
    print(f.filename())
    print(f.isfirstline())
    print(f.lineno())
    print(f.filelineno())
    for i in f:
        print(i,end='')

print('\n______________________________________')
with fileinput.input(files=('test.xml','test2.xml')) as f:
    line=f.readline()
    print(line)

import os
#os.rename('test.xml','t.txt')
#os.remove('somefilename.txt')
#os.rmdir('dirurl')
print(os.listdir())#returns a list of all files and folders

#f.seek(offset, whence)   ####
#هر وقت پارامتر دوم 0 باشد به معنی آفست از اول فایل است و اگر 1 باشد به معنی از مکان جاری کرسر و اگر 2 باشد به معنی آفست از آخر


import tempfile
print('tempfile.gettempdir():', tempfile.gettempdir())
temp = tempfile.TemporaryFile('w+')
print('temp.name:', temp.name)
print('temp.mode:', temp.mode)
temp.write('Hello world!')
temp.seek(0)
line = temp.readline()
print('line:', line)

from pathlib import Path
print('Create Path object for current directory')
p = Path('.')
p2 = Path(r'c:\Users')
print('p:', p)  #
print('p.exists():', p.exists()) # check dir exists()
print('p.is_dir():', p.is_dir()) # check the path is dir or not
print('p.is_file():', p.is_file()) # chek the path is file or not
print('p.absolute():', p.absolute())  # print(os.getcwd())

p = Path.cwd()
print('Set up new directory')
newdir = p / 'test'
print('Check to see if newdir exists')
print('newdir.exists():', newdir.exists())
print('Create new dir')
if(not newdir.exists()):
    newdir.mkdir()
print('newdir.exists():', newdir.exists())

c=newdir/'temp3'
print('c is this:', c)
c=newdir.joinpath('temp3')
print('c is this:', c)

try:
    newdir.rmdir()
except WindowsError as e:
    print(e)

p = Path('./new.py')
for i in p.glob('*'):
    print(i)

print(newdir.name)



import decimal
import json
from collections import deque
from datetime import datetime
import turtle
import sys
from math import pi

year = 1990
month = 11
day = 12
hour = 12
second = 30
minute = 11
if 1900 < year < 2100 and 1 <= month <= 12 \
        and 1 <= day <= 31 and 0 <= hour < 24 \
        and 0 <= minute < 60 and 0 <= second < 60:  # Looks like a valid date
    print('this date is valid')

# implicit line join
month_names = ['Januari', 'Februari', 'Maart',  # These are the
               'April', 'Mei', 'Juni',  # Dutch names
               'Juli', 'Augustus', 'September',  # for the months
               'Oktober', 'November', 'December']  # of the year

# keywords
# False      await      else       import     pass
# None       break      except     in         raise
# True       class      finally    is         return
# and        continue   for        lambda     try
# as         def        from       nonlocal   while
# assert     del        global     not        with
# async      elif       if         or         yield

# integers--------------------------------------------------
int1 = 2147483647
print(int1)
long1 = 79228162514264337593543950336
print(long1)
underlined_int = 100_000_000_000
print(underlined_int)
octal1 = 0o177
print(octal1)
print(type(octal1))
binary1 = 0b_1110_0101
print(binary1)
hexadecimal1 = 0xdeadbeef
print(hexadecimal1)

# floor of a division
print("floor of two int", underlined_int // int1)

# remain of a division
print("remain of to int", underlined_int % int1)

# power
print("power of two int", 4 ** 8)

# floating points--------------------------------------------

float1 = 3.14
print(float1)
float1 = 10.
print(float1)
float1 = .001
print(float1)
float1 = 1e100
print(float1)
float1 = 3.14e-10
print(float1)
float1 = 0e0
print(float1)
float1 = 3.14_15_93
print(float1)

# round
print('round of 13.14546 is:', round(13.14546, 2))

# اعداد موهومی--------------------------------------------------
imaginary1 = 3.14j
print(imaginary1)
imaginary1 = 10.j
print(imaginary1)
imaginary1 = .001j
print(imaginary1)
imaginary1 = 1e100j
print(imaginary1)
imaginary1 = 3.14e-10j
print(imaginary1)
imaginary1 = 0e0j
print(imaginary1)
imaginary1 = 3.14_15_93j
print(imaginary1)

# strings---------------------------------------------------------
name = "Fred"
print(f"He said his name is {name!r}.")  # add a single quotation
print(f"He said his name is {name}.")
print(f"He said his name is {repr(name)}.")  # add a single quotation
print(f"He said his name is {name.upper()}")  # .lower is the lowercase func
print()
# strings index
print(name[0])
print(name[-1])  # last character
print(name[-4])  # first character
print(name[0:2])  # Fre
print(name[:2] + name[2:])  # name[0:2]+name[2:last]
print(name[-2:])  # name[last-1:last]
print(len(name))  # length of name

width = 10
precision = 4
value = decimal.Decimal("12.34567")
print(f"result: {value:{width}.{precision}}")  # تعیین طول و دقت عدد
today = datetime(year=2017, month=1, day=27)
print(f"{today:%B %d, %Y}")  # فرمت تاریخ
number = 1024
print(f"{number:#0x}")  # hexadecimal
newline = ord('\n')
print(f"newline: {newline}")  # new line ord
newline = ord('\a')
print(f"newline: {newline}")  # new line ord

# quotation
print('this is single quotation example', 'doesn\'t')
print('this is single quotation example', "\"yes\" they said.")
print('this is double quotation example', "doesn't")
print('this is double quotation example', '"yes" they said.')

# \n delimiter
print('this is \\n delimiter example', 'C:\some\name')
print('this is \\n delimiter exampler', r'C:\some\name')

# triple quot example
print("""\
Usage: thingy [OPTIONS]
     -h                        Display this usage message
     -H hostname               Hostname to connect to
""")

# plus and time(cross) operators on string
print('this is plus and time operators on strings: ', 5 * 'ui' + 'tr')

# string concatenation
print('strings concat with space: ', 'Py' 'thon')
print('strings concat with plus: ', 'Py' + 'thon')


# formatted strings cannot use as function document
def foo():
    f"Not a docstring"


print(foo.__doc__ is None)

#  format strings
print('We are the {} who say "{}!"'.format('knights', 'Ni'))
print('{0} and {1}'.format('spam', 'eggs'))

print('We are the {} who say "{}!"'.format('knights', 'Ni'))
print('{0} and {1}'.format('spam', 'eggs'))
print('This {food} is {adjective}.'.format(
    food='spam', adjective='absolutely horrible'))
print('The story of {0}, {1}, and {other}.'.format('Bill', 'Manfred', other='Georg'))

table = {'Sjoerd': 4127, 'Jack': 4098, 'Dcab': 8637678}
print('Jack: {0[Jack]:d}; Sjoerd: {0[Sjoerd]:d}; '
      'Dcab: {0[Dcab]:d}'.format(table))
# Jack: 4098; Sjoerd: 4127; Dcab: 8637678
print('Jack: {Jack:d}; Sjoerd: {Sjoerd:d}; Dcab: {Dcab:d}'.format(**table))
# Jack: 4098; Sjoerd: 4127; Dcab: 8637678

print(vars())  # returns a dictionary containing all variables

for x in range(1, 11):
    print('{0:2d} {1:3d} {2:4d} {3:5d}'.format(x, x * x, x * x * x, x * x * x * x))

# equals to
for x in range(1, 11):
    print(repr(x).rjust(2), repr(x * x).rjust(3), end=' ')
    # Note use of 'end' on previous line
    print(repr(x * x * x).rjust(4), end=' ')
    print(repr(x * x * x * x).rjust(5))

# zfill()
print('12'.zfill(5))
print('-12.34'.zfill(7))
print('-12.34'.zfill(2))

# flow controls
# if--------------------------------------------------------------------------------

b = 3
a = 2
c = 1
if b > 0:
    print(a)
elif b < 0:
    print(b)
else:
    print(c)

# for-------------------------------------------------------------------------------
letters = ['a', 'b', 'c', 'd', 'e', 'f', 'g']
# foreach
for letter in letters:
    print(letter)

for letter in letters.copy():
    if letter == 'a':
        letters.append('x')

print(letters)

# range function
print('range(5)')
for i in range(5):
    print(i)

print('range(5, 10)')
for j in range(5, 10):
    print(j)

print('range(0, 10, 3)')
for j in range(0, 10, 3):
    print(j)

print('range(-10, -100, -30)')
for k in range(-10, -100, -30):
    print(k)

for i in range(len(letters)):
    print(i, letters[i])

# sum and list on range
print(sum(range(4)))
print(list(range(4)))

# break and else for a 'for loop'
for n in range(2, 100):
    t = n // 2
    for x in range(2, n):
        if n % x == 0:
            print(n, 'equals', x, '*', n // x)
            break
    else:
        # loop fell through without finding a factor
        print(n, 'is a prime number')

# continue
for num in range(2, 10):
    if num % 2 == 0:
        print("Found an even number", num)
        continue
    print("Found a number", num)


# pass
# an endless loop
# while True:
# pass

def initlog(*args):
    pass  # Remember to implement this!


# unpacking args-----------------------------------------------------
list(range(3, 6))  # [3, 4, 5]
args = [3, 6]
list(range(*args))  # [3, 4, 5]


# lambda ------------------------------------------------------------
def make_incrementor(n):
    return lambda x: x + n


f = make_incrementor(42)
print(f(0))


def powerto10(n):
    return lambda x: x ** n


f = powerto10(10)
print(f(2))

pairs = [(1, 'one'), (2, 'two'), (3, 'three'), (4, 'four')]
pairs.sort(key=lambda pair: pair[1])
print(pairs)
pairs.sort(key=lambda pair: len(pair[1]))
print(pairs)
pairs.sort(key=lambda pair: pair[0])
print(pairs)

a = [10, 'number', 11.2]
a.sort(key=lambda x: x.__class__.__name__)
print(a)


# functions----------------------------------------------------------
def fib(n):  # write Fibonacci series up to n
    """Print a Fibonacci series up to n."""
    a, b = 0, 1
    while a < n:
        print(a, end=' ')
        a, b = b, a + b
    print()


fib(2000)
func = fib
func(2000)


# default arguments
def ask_ok(prompt, retries=4, reminder='Please try again!'):
    ok = input(prompt)  # input
    if ok in ('y', 'ye', 'yes'):
        return True
    if ok in ('n', 'no', 'nop', 'nope'):
        return False
    retries = retries - 1
    if retries < 0:
        raise ValueError('invalid user response')
    print(reminder)


ask_ok('are you ok', 2, 'ok error')


def f(a, L=[]) -> str:
    L.append(a)
    return L


print(f(1))
print(f(2))
print(f(3))

print(f.__annotations__)

# List-------------------------------------------------------
# cut a list
letters = ['a', 'b', 'c', 'd', 'e', 'f', 'g']
print(letters[2:5])
letters[2:5] = []
print(letters)
print(len(letters))
# nested lists
x = [['a', 'b', 'c'], [1, 2, 3]]
print(x)
print(x[1])
print(x[1][1])
# swap to or more variables
a, b, c = 1, 2, 3
print(a, b, c)
a, b, c = b, c, a
print(a, b, c)

squares = [1, 4, 9, 16, 25]
print(squares, squares[4])
print(squares[-3:])  # prints 9, 16, 25
print(squares + [36, 49, 64, 81, 100])  # prints 1,4,9,...,100
# strings are immutable and we can't change name[i]
# but lists are mutable and we can change squares[i]

squares.append(36)
letters.clear()  # removes all items
print(letters)
letters.extend(['1', '2', '3'])  # equals to letters[len(letters):]=['1','2','3']
print(letters)
letters.insert(3, 'o')
letters.append('r')
letters.remove('2')
print(letters)
letters.pop()  # removes last item and returns it
print(letters)
letters.pop(2)  # removes item with index 2 and returns it
print(letters)

print(letters.count('1'))  # returns count of '1'
print(letters.index('1'))  # returns index of '1'
print(letters.index('1', 0, 4))  # returns index of '1' in letters[2:4] if exists

letters.reverse()  # reverses the order of elements
letters.sort()  # sorts the elements
letters.copy()  # copies the list

print(letters[0:4:2])  # third part indicates step count

# stack
# use a list as stack with functions pop() and append()

# queue
queue = deque(["Eric", "John", "Michael"])  # do not forget to import collections.deque
queue.append('ali')
queue.popleft()
print(queue)

# list comprehensions
squares2 = list(map(lambda rr: rr ** 2, range(10)))
print(squares2)

squares3 = [x ** 2 for x in range(10)]
print(squares3)

combos = [(x, y) for x in [1, 2, 3] for y in [3, 1, 4] if x != y]
print(combos)

combs = []
for x in [1, 2, 3]:
    for y in [3, 1, 4]:
        if x != y:
            combs.append((x, y))

print(combs)

vector = [2, 1, -2, 5]
vector2 = [x * 2 for x in vector]
vector3 = [x for x in vector if x >= 0]
vector4 = [abs(x) for x in vector]
print(vector2)
print(vector3)
print(vector4)

freshFruit = ['  banana', '  loganberry ', 'passion fruit  ']
temp = [weapon.strip() for weapon in freshFruit]  # you can call a function on each element
print(temp)

temp = [(x, x ** 2) for x in range(6)]  # you can create a two tuples
print(temp)

# flatten a 2d list
vec = [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
print([num for elem in vec for num in elem])

# complex expressions and nested functions
print([str(round(pi, i)) for i in range(1, 6)])

# zip() function - zip function
x = [1, 2, 3]
y = [4, 5]
z = [6, 7, 8, 9]
print(list(zip(x, y, z)))

# nested list comprehensions
matrix = [
    [1, 2, 3, 4],
    [5, 6, 7, 8],
    [9, 10, 11, 12],
]
print([[row[i] for row in matrix] for i in range(4)])  # [[1, 5, 9], [2, 6, 10], [3, 7, 11], [4, 8, 12]]

transp = []
for i in range(4):
    transp.append([row[i] for row in matrix])  # [[1, 5, 9], [2, 6, 10], [3, 7, 11], [4, 8, 12]]

print(transp)

print(list(zip(*matrix)))

# del statement
a = [-1, 1, 66.25, 333, 333, 1234.5]
del a[0]
print(a)
del a[2:4]
print(a)
del a[:]
print(a)
a = [-1, 1, 66.25, 333, 333, 1234.5]
del a
# print(a) error occurs

# tuples--------------------------------------------------
t = 12345, 54321, 'hello!'
print(t[0])
print(t)
u = t, (1, 2, 3, 4, 5)
print(u)

# t[0] = 88888 ## occurs an error because tuples are immutable
v = ([1, 2, 3], [3, 2, 1])
print(v)
v[1][1] = 1  # doesn't occur an error because tuples can take immutable objects
print(v[1][1])

empty = ()
singleton = 'hello',
print(len(empty))
print(len(singleton))
t = 12345, 54321, 'hello!'
x, y, z = t
print(z)

# Sets---------------------------------------------------
basket = {'apple', 'orange', 'apple', 'pear', 'orange', 'banana'}
print(basket)

print('orange' in basket)  # fast membership testing

print('crabgrass' in basket)

# Demonstrate set operations on unique letters from two words

a = set('abracadabra')
b = set('alacazam')
print(a)  # unique letters in a
print(a - b)  # letters in a but not in b
print(a | b)  # letters in a or b or both
print(a & b)  # letters in both a and b
print(a ^ b)  # letters in a or b but not both

# Similarly to list comprehensions, set comprehensions are also supported
a = {x for x in 'abracadabra' if x not in 'abc'}
print(a)

# Dictionaries---------------------------------------------
tel = {'jack': 4098, 'sape': 4139}
tel['guido'] = 4127
print(tel)
print(tel['jack'])
del tel['sape']
tel['irv'] = 4127
print(tel)
list(tel)
sorted(tel)
print('guido' in tel)
print('jack' not in tel)
print(dict([('sape', 4139), ('guido', 4127), ('jack', 4098)]))
newdict = {x: x ** 2 for x in (2, 4, 6)}
print(newdict)
newdict = dict(sape=4139, guido=4127, jack=4098)
print(newdict)

# files--------------------------------------------
# with means open file and after end close
with open('workfile') as f:  # read entire file 'workfile' 'r' tag for read
    print(f.tell())  # returns the position of the cursor
    # f.seek(offset, whence)
    print(f.seek(12))  # seek the cursor to giving position
    read_data = f.read()
    print(f.tell())
    read_data2 = f.read()  # returns empty string
    print(read_data)
    print(read_data2)
    # print(f.seek(-3))

print(f.close())
print('///')

# read line
with open('workfile') as f:
    for line in f:  # read line of a file 'workfile' is dir
        print(line, end='')  # end='' is for preventing of end-of-line
# f.close()

# write
with open('workfile', 'w') as f:  # 'w' tag for write 'x' for exec 'a' for append
    # https://docs.python.org/3/library/functions.html#open
    f.write('This is a test\n')

# json--------------------------------------------------

print(json.dumps([1, 'simple', 'list']))
x = json.dumps([1, 'simple', 'list'])
with open('workfile', 'w') as f:
    json.dump(x, f)

with open('workfile', 'r') as f:
    x = json.load(f)

print(x)

yd = json.dumps({'ali': 'saifi', 'mohsen': 'saifi'})
print(yd)


# namespaces ---------------------------------------------
# global means in global scope
# nonlocal means in enclosing scope نزدیک ترین والد
def scope_test():
    def do_local():
        spam = "local spam"

    def do_nonlocal():
        nonlocal spam
        spam = "nonlocal spam"

    def do_global():
        global spam
        spam = "global spam"

    spam = "test spam"
    do_local()
    print("After local assignment:", spam)
    do_nonlocal()
    print("After nonlocal assignment:", spam)
    do_global()
    print("After global assignment:", spam)


scope_test()
print("In global scope:", spam)


# classes----------------------------------------------
class MyClass:
    """A simple example class"""
    staticVar = 2  # static object

    def f(self):
        return 'hello world'

    # static example
    @staticmethod
    def f2():
        return 'hello world'

    def printPrivate(self):
        print(self._privatevalue)

    def __init__(self, tempList):
        self.data = tempList
        self.i = 12345
        self._privatevalue = 2  # private value starts with '_'

    # def __init__(self):
    #     self.data = [0, 0]


x = MyClass([1, 2])
print(x.i)
print(x.f())
x.printPrivate()
print(x.data)
del x.data


# print(x._privatevalue)


# print(x.data) this will raise an error


# Function defined outside the class
def f1(self, x, y):
    return min(x, x + y)


class C:
    f = f1

    def g(self):
        return 'hello world'

    h = g


c = C()
print(c.f(1, -1))
print(c.h())


class D(C):
    data2 = 'this is inheritance'


d = D()
print(d.g())


class GG(C, MyClass):
    def __init__(self):
        g = 0

    def g(self):  # overrides C.g()
        return 'hello world2'


gg = GG()
print(gg.g())


# Structure --------------------------
class Employee:
    pass


john = Employee()  # Create an empty employee record

# Fill the fields of the record
john.name = 'John Doe'
john.dept = 'computer lab'
john.salary = 1000

print(john.salary)

# iterators
for element in [1, 2, 3]:
    print(element)
for element in (1, 2, 3):
    print(element)
for key in {'one': 1, 'two': 2}:
    print(key)
for char in "123":
    print(char)
for line in open("workfile"):
    print(line, end='')

s = 'abc'
it = iter(s)
print(it)
print(next(it))
print(next(it))
print(next(it))


class Reverse:
    """Iterator for looping over a sequence backwards."""

    def __init__(self, data):
        self.data = data
        self.index = len(data)

    def __iter__(self):
        return self

    def __next__(self):
        if self.index == 0:
            raise StopIteration
        self.index = self.index - 1
        return self.data[self.index]


for char in Reverse('spam'):
    print(char)


# generators
def reverse(data):
    for index in range(len(data) - 1, -1, -1):
        yield data[index]


for char in reverse('golf'):
    print(char)

# Generator expressions
sum(i * i for i in range(10))
xvec = [10, 20, 30]
yvec = [7, 5, 3]
sum(x * y for x, y in zip(xvec, yvec))
data = 'golf'
list(data[i] for i in range(len(data) - 1, -1, -1))

# turtle----------------------------------------------------
# turtle.color('red', 'yellow')
# turtle.begin_fill()
# while True:
#     turtle.forward(200)
#     turtle.left(170)
#     if abs(turtle.pos()) < 1:
#         break
# turtle.end_fill()
# turtle.done()

# Exception handling ------------------------------------------
while True:
    try:
        x = int(input("Please enter a number: "))
        break
    except (RuntimeError, TypeError, NameError, ValueError):
        print("Oops!  That was no valid number.  Try again...")
        print()


class B(Exception):
    pass


class C(B):
    pass


class D(C):
    pass


for cls in [B, C, D]:
    try:
        raise cls()
    except D:
        print("D")
    except C:
        print("C")
    except B:
        print("B")

try:
    f = open('myfile.txt')
    s = f.readline()
    i = int(s.strip())
except OSError as err:
    print("OS error: {0}".format(err))
except ValueError:
    print("Could not convert data to an integer.")
except:
    print("Unexpected error:", sys.exc_info()[0])
    raise

# else statement in try runs if no exception 'it is very useful really'
for arg in sys.argv[1:]:
    try:
        f = open(arg, 'r')
    except OSError:
        print('cannot open', arg)
    else:
        print(arg, 'has', len(f.readlines()), 'lines')
        f.close()

# raise statement
try:
    raise Exception('spam', 'eggs')
except Exception as inst:
    print(type(inst))  # the exception instance
    print(inst.args)  # arguments stored in .args
    print(inst)  # __str__ allows args to be printed directly,


# finally example
def bool_return():
    try:
        return True
    finally:
        return False


print(bool_return())
try:
    raise NameError('HiThere')  # this raises an exception
except NameError:
    print('An exception flew by!')
    raise



import os
import pathlib

###_________________________________________________
#Delete a directory
def deleteDir(path1):
        if(path1.is_file()):
            os.remove(path1)
        else:
            try:
                path1.rmdir()
            except WindowsError:
                for i in os.listdir(path1):
                    deleteDir(path1/i)
                path1.rmdir()

# or u can use shutil.rmtree(dirstr)


import os
import shutil
import glob
import sys
import argparse
import re
import math
import random
import statistics

print(os.getcwd())  # get current dir
# make a directory
# os.system('mkdir today')
os.chdir('./today')  # change directory
print(os.getcwd())
print(dir(os))  # returns all module funcs
print(help(os))  # returns all module help
os.chdir('../../')
# copy a file
try:
    shutil.copyfile('workfile', 'archive.db')
except IOError:
    print('this file exists and not copied')
# move a file
try:
    shutil.move('archive.db', 'd:\\')
except IOError:
    print("this file exists and didn't moved")

os.chdir('d:\\')  # change directory
print(glob.glob('*.exe'))
print(sys.argv)  # arg[0] is the current executed file
print(sys.stderr.write('Warning, log file not found starting a new one\n'))
print(re.findall(r'\bf[a-z]*', 'which foot or hand fell fastest'))
print(re.sub(r'(\b[a-z]+) \1', r'\1', 'cat in the the hat'))
print('tea for too'.replace('too', 'two'))
print(math.cos(math.pi / 4))
print(math.log(1024, 2))
print(random.choice(['apple', 'pear', 'banana']))
print(random.sample(range(100), 10))
print(random.random())    # random float
print(random.randrange(6))    # random integer chosen from range(6)

data = [2.75, 1.75, 1.25, 0.25, 0.5, 1.25, 3.5]
print(statistics.mean(data))
print(statistics.median(data))
print(statistics.variance(data))

parser = argparse.ArgumentParser(prog='top',
                                 description='Show top lines from each file')
parser.add_argument('filenames', nargs='+')
parser.add_argument('-l', '--lines', type=int, default=10)
args = parser.parse_args()
print(args)
# python top.py --lines=5 alpha.txt beta.txt
list=args.filenames
for file1 in list:
    with open(file1) as f:
        for i in range(args.lines):
            print(f.readline(),end=' ')

sys.exit()  # exits from script



'''import argparse


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Echo your input')
    parser.add_argument('message',
                        help='Message to echo')

    parser.add_argument('--twice', '-t',
                        help='Do it twice',
                        action='store_true')

    args = parser.parse_args()

    print(args.message)
    if args.twice:
        print(args.message)'''


import argparse

def sail():
    ship_name = 'Your ship'
    print(f"{ship_name} is setting sail")

def list_ships():
    ships = ['John B', 'Yankee Clipper', 'Pequod']
    print(f"Ships: {','.join(ships)}")

def greet(greeting, name):
    message = f'{greeting} {name}'
    print(message)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Maritime control')

    parser.add_argument('--twice', '-t',
                        help='Do it twice',
                        action='store_true')

    subparsers = parser.add_subparsers(dest='func')

    ship_parser = subparsers.add_parser('ships',
                                        help='Ship related commands')
    ship_parser.add_argument('command',
                             choices=['list', 'sail'])

    sailor_parser = subparsers.add_parser('sailors',
                                          help='Talk to a sailor')
    sailor_parser.add_argument('name',
                               help='Sailors name')
    sailor_parser.add_argument('--greeting', '-g',
                               help='Greeting',
                               default='Ahoy there')
    args = parser.parse_args()
    if args.func == 'sailors':
        greet(args.greeting, args.name)
    elif args.command == 'list':
        list_ships()
    else:
        sail()


import calendar
print(calendar.weekheader(2))  # returns weekheader with length 2
print(calendar.firstweekday())  # prints 0 for monday
print(calendar.month(2020, 1))  # returns calendar of a month in calendar format
print(calendar.monthcalendar(2020, 1))  # returns calendar of a month in list format
print(calendar.calendar(2020))  # returns calendar of a year in calendar format
for i in range(1, 12):  # returns calendar of a year in list format
    print(calendar.monthcalendar(2020, i))

print(calendar.weekday(2020, 2, 28))  # returns weekday of a date
print(calendar.isleap(2020))  # returns true if 2020 is leap year
print(calendar.leapdays(2000, 2021))  # not included 2021




import threading
from threading import Thread
from time import sleep


def simple_worker():
    print('hello\n')


t1 = Thread(target=simple_worker)
t1.start()
print(t1.getName())
print(t1.ident)
print(t1.is_alive())


print(threading.active_count())
th1=threading.current_thread() #returns current thread
thId=threading.get_ident() # return current thread ID

threading.enumerate() # list of alive threads
#The list includes daemon threads, dummy thread objects created by current_thread() and the main thread

mainThread=threading.main_thread() # returns main thread



'''def worker():
    for i in range(0, 10):
        print('.', end='', flush=True)
        sleep(1)


print('Starting')

# Create read object with reference to worker function
t = Thread(target=worker)

# Start the thread object
t.start()

# Wait for the thread to complete
t.join()
print('\nDone')'''

def worker(msg):
    for i in range(0, 10):
        print(msg, end='', flush=True)
        sleep(1)
print('Starting')
t1 = Thread(target=worker,name='thread 1', args='A') # arguments must be a tuple
t2 = Thread(target=worker,name='thread 2', args='B')
t3 = Thread(target=worker, args='C')
t1.start()
t2.start()
t3.start()
print('Done')

print(threading.active_count())
th1=threading.current_thread() #returns current thread
thId=threading.get_ident() # return current thread ID
print(th1.name , thId)
print(threading.enumerate()) # list of alive threads
#The list includes daemon threads, dummy thread objects created by current_thread() and the main thread

mainThread=threading.main_thread()
print(mainThread.ident)


class WorkerThread(Thread):
    def __init__(self,daemon=None,target=None , name=None):
        super().__init__(daemon=daemon,target=target,name=name)

    def run(self):
        for i in range(0, 10):
            print('.', end='', flush=True)
            sleep(1)

print('Starting')
t = WorkerThread()
t.start()
print('\nDone')

### daemon thread
def worker(msg):
    for i in range(0, 10):
        print(msg, end='', flush=True)
        sleep(1)
print('Starting')
# Create a daemon thread
d = Thread(daemon=True, target=worker, args='C')
d.start()
sleep(5)
print('Done')

from threading import Timer
def hello():
    print('hello')
print('Starting')
t = Timer(5, hello)
t.start()
print('Done')

from threading import Thread, local, currentThread
from random import randint


def show_value(data):
    try:
        val = data.value
    except AttributeError:
        print(currentThread().name, ' : No value yet')
    else:
        print(currentThread().name, ' : value =', val)


def worker(data):
    show_value(data)
    data.value = randint(1, 100)
    show_value(data)


print(currentThread().name, ' : Starting')
local_data = local()
show_value(local_data)
for i in range(2):
    t = Thread(name='W' + str(i), target=worker, args=[local_data])
    t.start()
show_value(local_data)
print(currentThread().name, ' : Done')



import pymysql

connection = pymysql.connect('localhost', 'root', '9125432834@Ali', 'ali')
cursor = connection.cursor()
# cursor.execute("CREATE database ali")

'''try:
    cursor.execute("create TABLE log (message VARCHAR(100) NOT NULL, id INT(6) NOT NULL)")
    connection.commit()
except:
    connection.rollback()
'''

# cursor.execute("drop TABLE log")
# connection.commit()
'''try:
    cursor.execute("Insert into log (message,id) values ('third log3',89)")
    connection.commit()
except:
    connection.rollback()
'''

'''try:
    cursor.execute("UPDATE log SET message = 'denise@my.com' WHERE id = 89")
    connection.commit()
except:
    connection.rollback()
'''

'''try:
    cursor.execute("DELETE FROM log WHERE id = 89")
    connection.commit()
except:
    connection.rollback()
'''

cursor.execute("select * from log")

data = cursor.fetchall()
for row in data:
    message, id = row
    print('row:', message, ' ', id)

print(cursor.description)
print(cursor.rowcount)
connection.close()



import hashlib
import sys

secret = "This is the password or document text"
bsecret = secret.encode()
print(bsecret)
m = hashlib.md5()
print(m)
m.update(bsecret)

print(m.digest())
print(hashlib.md5(bsecret).digest())


from cryptography.fernet import Fernet

key = Fernet.generate_key()

print(key)

f = Fernet(key)

message = b"Secrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go hereSecrets go here"


encrypted = f.encrypt(message)

print(encrypted)
print(f.decrypt(encrypted))
print(sys.getsizeof(message),sys.getsizeof(encrypted))

from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa, padding

private_key = rsa.generate_private_key(public_exponent=65537,
                                       key_size=4096,
                                       backend=default_backend())

public_key = private_key.public_key
print(public_key)

public_key = private_key.public_key()
print(public_key)

message = b"More secrets go here"

encrypted = public_key.encrypt(message,
                               padding.OAEP(mgf=padding.MGF1(algorithm=hashes.SHA512()),
                                            algorithm=hashes.SHA512(),
                                            label=None))

print(encrypted)

decrypted = private_key.decrypt(encrypted,
                                padding.OAEP(mgf=padding.MGF1(algorithm=hashes.SHA512()),
                                             algorithm=hashes.SHA512(),
                                             label=None))

print(decrypted)



from time import sleep

from bs4 import BeautifulSoup
import requests

while True:
    req = requests.get('https://www.tgju.org/')
    #print(req.content)
    soup = BeautifulSoup(req.content, 'html.parser')
    findlist = soup.findAll(class_='info-price')
    print(findlist[5].get_text())
    print(findlist[8].get_text())
    sleep(10)




import numpy

# Import matplotlib colour map
from pythonw.matplotlib import cm as colourmap
# Required for £D Projections
# Provide access to numpy functions


def line_graph():
    # Set up the data
    x = [0, 1, 2, 3, 4, 5, 6]
    y = [0, 2, 6, 14, 30, 43, 75]
    # Set the axes headings
    pyplot.ylabel('Speed', fontsize=12)
    pyplot.xlabel('Time', fontsize=12)
    # Set the title
    pyplot.title("Speed v Time")
    # Plot and display the graph
    # Using blue circles for markers ('bo')
    # and a solid line ('-')
    pyplot.plot(x, y, 'bo-')
    '''
    b,g,r,c,m,y,k,w
    blue, green, red, cyan, magenta, yellow, black, white

    ‘.’ point marker
    ‘,’ pixel marker
    ‘o’ circle marker
    ‘v’ triangle_down marker
    ‘^’ triangle_up marker
    ‘ < ’ triangle_left marker
    ‘ > ’ triangle_right marker
    ‘s’ square marker
    ‘p’ pentagon marker
    ‘*’ star marker
    ‘h’ hexagon1 marker
    ‘ + ’ plus marker
    ‘x’ x marker
    ‘D’ diamond marker
    ‘-’ solid line style
    ‘–’ dashed line style
    ‘-.’ dash-dot line style
    ‘:’ dotted line style
    '''
    pyplot.show()


def scatter_graph():
    # Create data
    riding = ((17, 18, 21, 22, 19, 21, 25, 22, 25, 24),
              (3, 6, 3.5, 4, 5, 6.3, 4.5, 5, 4.5, 4))
    swimming = ((17, 18, 20, 19, 22, 21, 23, 19, 21, 24),
                (8, 9, 7, 10, 7.5, 9, 8, 7, 8.5, 9))
    sailing = ((31, 28, 29, 36, 27, 32, 34, 35, 33, 39),
               (4, 6.3, 6, 3, 5, 7.5, 2, 5, 7, 4))
    # Plot the data
    pyplot.scatter(x=riding[0], y=riding[1], c='red', marker='o',
                   label='riding')
    pyplot.scatter(x=swimming[0], y=swimming[1], c='green',
                   marker='^', label='swimming')
    pyplot.scatter(x=sailing[0], y=sailing[1], c='blue',
                   marker='*', label='sailing')
    # Configure graph
    pyplot.xlabel('Age')
    pyplot.ylabel('Hours')
    pyplot.title('Activities Scatter Graph')
    pyplot.legend()
    pyplot.show()
    # Display the chart

def scatter_line_graph():
    x = (5, 5.5, 6, 6.5, 7, 8, 9, 10)
    y = (120, 115, 100, 112, 80, 85, 69, 65)
    # Generate the scatter plot
    pyplot.scatter(x, y)
    # Generate the trend line
    z = numpy.polyfit(x, y, 1)
    p = numpy.poly1d(z)
    pyplot.plot(x, p(x), 'r')
    # Display the figure
    pyplot.show()

def pie_graph():
    labels = ('Python', 'Java', 'Scala', 'C#')
    sizes = [45, 30, 15, 10]
    pyplot.pie(sizes,
               labels=labels,
               autopct='%1.f%%',
               counterclock=False,
               startangle=90)
    pyplot.show()

def pie_graph2():
    labels = ('Python', 'Java', 'Scala', 'C#')
    sizes = [45, 30, 15, 10]
    # only "explode" the 1st slice (i.e. 'Python')
    explode = (0.1, 0, 0, 0)
    pyplot.pie(sizes,
               explode=explode,
               labels=labels,
               autopct='%1.f%%',
               shadow=True,
               counterclock=False,
               startangle=90)
    pyplot.show()


def bar_graph():
    # Set up the data
    labels = ('Python', 'Scala', 'C#', 'Java', 'PHP')
    index = (1, 2, 3, 4, 5)  # provides locations on x axis
    sizes = [45, 10, 15, 30, 22]
    # Set up the bar chart
    pyplot.bar(index, sizes, tick_label=labels)
    # Configure the layout
    pyplot.ylabel('Usage')
    pyplot.xlabel('Programming Languages')
    # Display the chart
    pyplot.show()

def bar_horizontal_graph():
    # Set up the data
    labels = ('Python', 'Scala', 'C#', 'Java', 'PHP')
    index = (1, 2, 3, 4, 5)  # provides locations on x axis
    sizes = [45, 10, 15, 30, 22]
    # Set up the bar chart
    pyplot.barh(index, sizes, tick_label=labels)
    # Configure the layout
    pyplot.ylabel('Usage')
    pyplot.xlabel('Programming Languages')
    # Display the chart
    pyplot.show()

def bar_colored_graph():
    # Set up the data
    labels = ('Python', 'Scala', 'C#', 'Java', 'PHP')
    index = (1, 2, 3, 4, 5)  # provides locations on x axis
    sizes = [45, 10, 15, 30, 22]
    # Set up the bar chart
    pyplot.bar(index, sizes, tick_label=labels, color=('red',
                                                          'green', 'blue', 'yellow', 'orange'))
    # Configure the layout
    pyplot.ylabel('Usage')
    pyplot.xlabel('Programming Languages')
    # Display the chart
    pyplot.show()

def bar_stacked_graph():
    # Set up the data
    labels = ('Python', 'Scala', 'C#', 'Java', 'PHP')
    index = (1, 2, 3, 4, 5)
    web_usage = [20, 2, 5, 10, 14]
    data_science_usage = [15, 8, 5, 15, 2]
    games_usage = [10, 1, 5, 5, 4]
    # Set up the bar chart
    pyplot.bar(index, web_usage, tick_label=labels, label='web')
    pyplot.bar(index, data_science_usage, tick_label=labels,
               label='data science', bottom=web_usage)
    web_and_games_usage = [web_usage[i] + data_science_usage[i]
                           for i in range(0, len(web_usage))]
    pyplot.bar(index, games_usage, tick_label=labels,
               label='games', bottom=web_and_games_usage)
    # Configure the layout
    pyplot.ylabel('Usage')
    pyplot.xlabel('Programming Languages')
    pyplot.legend()
    # Display the chart
    pyplot.show()

def bar_grouped_graph():
    BAR_WIDTH = 0.35
    # set up grouped bar charts
    teama_results = (60, 75, 56, 62, 58)
    teamb_results = (55, 68, 80, 73, 55)
    # Set up the index for each bar
    index_teama = (1, 2, 3, 4, 5)
    index_teamb = [i + BAR_WIDTH for i in index_teama]
    # Determine the mid point for the ticks
    ticks = [i + BAR_WIDTH / 2 for i in index_teama]
    tick_labels = ('Lab 1', 'Lab 2', 'Lab 3', 'Lab 4', 'Lab 5')
    # Plot the bar charts
    pyplot.bar(index_teama, teama_results, BAR_WIDTH, color='b',
               label='Team A')
    pyplot.bar(index_teamb, teamb_results, BAR_WIDTH, color='g',
               label='Team B')
    # Set up the graph
    pyplot.xlabel('Labs')
    pyplot.ylabel('Scores')
    pyplot.title('Scores by Lab')
    pyplot.xticks(ticks, tick_labels)
    pyplot.legend()
    # Display the graph
    pyplot.show()

def subplots():
    t = range(0, 20)
    s = range(30, 10, -1)
    # Set up the grid of subplots to be 2 by 2
    grid_size = '22'
    # Initialize a Figure
    figure = pyplot.figure()
    # Add first subplot
    position = grid_size + '1'
    print('Adding first subplot to position', position)
    axis1 = figure.add_subplot(position)
    axis1.set(title='subplot(2,2,1)')
    axis1.plot(t, s)
    # Add second subplot
    position = grid_size + '2'
    print('Adding second subplot to position', position)
    axis2 = figure.add_subplot(position)
    axis2.set(title='subplot(2,2,2)')
    axis2.plot(t, s, 'r-')
    # Add third subplot
    position = grid_size + '3'
    print('Adding third subplot to position', position)
    axis3 = figure.add_subplot(position)
    axis3.set(title='subplot(2,2,3)')
    axis3.plot(t, s, 'g-')
    # Add fourth subplot
    position = grid_size + '4'
    print('Adding fourth subplot to position', position)
    axis4 = figure.add_subplot(position)
    axis4.set(title='subplot(2,2,4)')
    axis4.plot(t, s, 'y-')
    # Display the chart
    pyplot.show()

def treeD_graph():
    # Make the data to be displayed
    x_values = numpy.arange(-6, 6, 0.3)
    y_values = numpy.arange(-6, 6, 0.3)
    # Generate coordinate matrices from coordinate vectors
    x_values, y_values = numpy.meshgrid(x_values, y_values)
    # Generate Z values as sin of x plus y values
    z_values = numpy.sin(x_values + y_values)
    # Obtain the figure object
    figure = pyplot.figure()
    # Get the axes object for the 3D graph
    axes = figure.gca(projection='3d')
    # Plot the surface.
    surf = axes.plot_surface(x_values,
                             y_values,
                             z_values,
                             cmap=colourmap.coolwarm)
    # Add a color bar which maps values to colors.
    figure.colorbar(surf)
    # Add labels to the graph
    pyplot.title("3D Graph")
    axes.set_ylabel('y values', fontsize=8)
    axes.set_xlabel('x values', fontsize=8)
    axes.set_zlabel('z values', fontsize=8)
    # Display the graph
    pyplot.show()

#line_graph()
#scatter_graph()
#scatter_line_graph()
#pie_graph()
#pie_graph2()
#bar_graph()
#bar_horizontal_graph()
#bar_colored_graph()
#bar_stacked_graph()
#bar_grouped_graph()
#subplots()
treeD_graph()


from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()


setup(
    name="pythonw",
    version="3.0.3",
    author="ali sayfi",
    author_email="ali.35351@gmail.com.com",
    url="http://yobinet.ir",
    description="python instructions example",
    packages=find_packages(),
    long_description='long description',
    long_description_content_type='text/x-rst',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


import multiprocessing
import os
from time import sleep


def worker(msg):
    print('process id:', os.getpid())
    for i in range(0, 10):
        print(msg, end='')
        sleep(1)

print('Starting')
if __name__=='__main__':
    print('process id:', os.getpid())
    ctx = multiprocessing.get_context('spawn')
    p = ctx.Process(target=worker, args=('A',))
    '''or p = Process(target=worker, args=('A',),name='ali')'''
    p.start()

print('Done')

from multiprocessing import Pool
from time import sleep


def worker(x):
    print('In worker with: ', x)
    sleep(3)
    return x * x


if __name__ == '__main__':
    with Pool(processes=4) as pool:
        print(pool.map(worker, [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]))


if __name__== '__main__':
    with Pool(processes=4) as pool:
        for result in pool.imap_unordered(worker,[1,2,3,4,5,6,7,8,9]):
            print(result)